@@ -36,7 +36,6 @@
 #include "skey-popup.h"
 #include <libgnome/gnome-util.h> /* gnome_util_user_shell */
 #include <libgnome/gnome-url.h> /* gnome_url_show */
-#include <libgnomevfs/gnome-vfs-utils.h> /* gnome_vfs_make_uri_from_input */
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -90,6 +89,7 @@ enum {
 
 static void terminal_screen_init        (TerminalScreen      *screen);
 static void terminal_screen_class_init  (TerminalScreenClass *klass);
+static void terminal_screen_dispose     (GObject             *object);
 static void terminal_screen_finalize    (GObject             *object);
 static void terminal_screen_unrealize   (GtkWidget *widget);
 static void terminal_screen_size_allocate (GtkWidget *widget,
@@ -241,6 +241,48 @@ terminal_screen_size_allocate (GtkWidget *widget,
   gtk_widget_size_allocate (child, allocation);
 }
 
+static void
+terminal_screen_sync_settings (GtkSettings *settings,
+                               GParamSpec *pspec,
+                               TerminalScreen *screen)
+{
+  gboolean blink;
+
+  g_object_get (G_OBJECT (settings),
+                "gtk-cursor-blink", &blink,
+                NULL);
+
+  terminal_widget_set_cursor_blinks (screen->priv->term, blink);
+}
+
+static void
+terminal_screen_screen_changed (GtkWidget *widget, GdkScreen *previous_screen)
+{
+  GdkScreen *screen;
+  GtkSettings *settings;
+
+  screen = gtk_widget_get_screen (widget);
+  if (previous_screen != NULL &&
+      (screen != previous_screen || screen == NULL)) {
+    settings = gtk_settings_get_for_screen (previous_screen);
+    g_signal_handlers_disconnect_matched (settings, G_SIGNAL_MATCH_DATA,
+                                          0, 0, NULL, NULL,
+                                          widget);
+  }
+
+  if (GTK_WIDGET_CLASS (terminal_screen_parent_class)->screen_changed) {
+    GTK_WIDGET_CLASS (terminal_screen_parent_class)->screen_changed (widget, previous_screen);
+  }
+
+  if (screen == previous_screen)
+    return;
+
+  settings = gtk_widget_get_settings (widget);
+  terminal_screen_sync_settings (settings, NULL, TERMINAL_SCREEN (widget));
+  g_signal_connect (settings, "notify::gtk-cursor-blink",
+                    G_CALLBACK (terminal_screen_sync_settings), widget);
+}
+
 static void
 terminal_screen_grab_focus (GtkWidget *widget)
 {
@@ -365,10 +407,12 @@ terminal_screen_class_init (TerminalScreenClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
 
+  object_class->dispose = terminal_screen_dispose;
   object_class->finalize = terminal_screen_finalize;
 
   widget_class->unrealize = terminal_screen_unrealize;
   widget_class->size_allocate = terminal_screen_size_allocate;
+  widget_class->screen_changed = terminal_screen_screen_changed;
   widget_class->size_request = terminal_screen_size_request;
   widget_class->map = terminal_screen_map;
   widget_class->grab_focus = terminal_screen_grab_focus;
@@ -437,14 +481,28 @@ terminal_screen_unrealize (GtkWidget *widget)
   GTK_WIDGET_CLASS (terminal_screen_parent_class)->unrealize (widget);
 }
 
+static void
+terminal_screen_dispose (GObject *object)
+{
+  TerminalScreen *screen;
+  GtkSettings *settings;
+
+  screen = TERMINAL_SCREEN (object);
+
+  settings = gtk_widget_get_settings (GTK_WIDGET (screen));
+  g_signal_handlers_disconnect_matched (settings, G_SIGNAL_MATCH_DATA,
+                                        0, 0, NULL, NULL,
+                                        screen);
+
+  G_OBJECT_CLASS (terminal_screen_parent_class)->dispose (object);
+}
+
 static void
 terminal_screen_finalize (GObject *object)
 {
   TerminalScreen *screen;
   GConfClient *conf;
 
-  screen = TERMINAL_SCREEN (object);
-
   conf = gconf_client_get_default ();
   
   if (screen->priv->gconf_connection_id)
@@ -572,9 +630,6 @@ terminal_screen_reread_profile (TerminalScreen *screen)
 
   update_color_scheme (screen);
 
-  terminal_widget_set_cursor_blinks (term,
-                                     terminal_profile_get_cursor_blink (profile));
-
   terminal_widget_set_audible_bell (term,
                                     !terminal_profile_get_silent_bell (profile));
 
